@@ -1,4 +1,19 @@
-import argparse, sys
+#
+# Copyright 2014-2025 Thomas Rabaix <thomas.rabaix@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License. You may obtain
+# a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+# License for the specific language governing permissions and limitations
+# under the License.
+
+import argparse
 
 class Command(object):
 
@@ -8,7 +23,6 @@ class Command(object):
     def execute(self, args, output):
         pass
 
-
 class CommandManager(object):
     def __init__(self, commands=None):
         self.commands = commands or {}
@@ -26,9 +40,9 @@ class CommandManager(object):
         self.commands[name] = (parser, command)
 
     def execute(self, argv, stdout):
-        argv.pop(0) # remove the script name
+        argv.pop(0)  # remove the script name
 
-        if len(argv) == 0: # no argument
+        if len(argv) == 0:  # no argument
             name = 'help'
             argv = []
         else:
@@ -43,8 +57,8 @@ class CommandManager(object):
 
         r = command.execute(arguments, stdout)
 
-        if r == None:
-           r = 0
+        if r is None:
+            r = 0
 
         return r
 
@@ -58,7 +72,6 @@ class HelpCommand(Command):
         pass
 
     def execute(self, args, output):
-        
         output.write("Commands available: \n")
         for name, (parser, command) in self.command_manager.commands.items():
             output.write(" > % -20s : %s \n" % (name, parser.description))
