@@ -1,9 +1,26 @@
-import ioc.loader, ioc.component, ioc.exceptions
-import os, datetime
+#
+# Copyright 2014-2025 Thomas Rabaix <thomas.rabaix@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License. You may obtain
+# a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+# License for the specific language governing permissions and limitations
+# under the License.
+
+import ioc.component
 
 class Extension(ioc.component.Extension):
     def load(self, config, container_builder):
-        container_builder.add('ioc.extra.event_dispatcher', ioc.component.Definition('ioc.event.Dispatcher'))
+        definition = ioc.component.Definition('ioc.event.Dispatcher', kwargs={
+            'logger': ioc.component.Reference('logger')
+        })
+        container_builder.add('ioc.extra.event_dispatcher', definition)
 
     def post_build(self, container_builder, container):
         dispatcher = container.get('ioc.extra.event_dispatcher')
@@ -25,4 +42,4 @@ class Extension(ioc.component.Extension):
                 else:
                     method = option['method']
 
-                dispatcher.add_listener(option['name'], method, option['priority'])
\ No newline at end of file
+                dispatcher.add_listener(option['name'], method, option['priority'])
