@@ -1,4 +1,18 @@
-# -*- coding: utf-8 -*-
+#
+# Copyright 2014-2025 Thomas Rabaix <thomas.rabaix@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License. You may obtain
+# a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+# License for the specific language governing permissions and limitations
+# under the License.
+
 """
     Resource locator
 
@@ -9,7 +23,7 @@ from os import path
 class ResourceNotFound(Exception):
     pass
 
-def split_resource_path(resource):
+def split_resource_path(resource: str) -> list[str]:
     """Split a path into segments and perform a sanity check.  If it detects
     '..' in the path it will raise a `TemplateNotFound` error.
     """
@@ -25,7 +39,7 @@ def split_resource_path(resource):
 
 
 class BaseLocator(object):
-    def locate(self, resource):
+    def locate(self, resource: str) -> str:
         raise ResourceNotFound(resource)
 
 
@@ -43,13 +57,8 @@ class FileSystemLocator(BaseLocator):
     """
 
     def __init__(self, searchpath):
-        
-        try:
-            if isinstance(searchpath, basestring):
-                searchpath = [searchpath]
-        except NameError:
-            if isinstance(searchpath, str):
-                searchpath = [searchpath]
+        if isinstance(searchpath, str):
+            searchpath = [searchpath]
 
         self.searchpath = list(searchpath)
 
@@ -76,20 +85,44 @@ class PackageLocator(BaseLocator):
     If the package path is not given, ``'resources'`` is assumed.
     """
 
-    def __init__(self, package_name, package_path='resources'):
-        from pkg_resources import ResourceManager, get_provider
-        self.manager = ResourceManager()
-        self.provider = get_provider(package_name)
+    def __init__(self, package_name: str, package_path: str = 'resources') -> None:
+        try:
+            # Python 3.9+
+            from importlib import resources
+            self.resources = resources
+        except ImportError:
+            # Fallback for older Python versions
+            import importlib_resources as resources
+            self.resources = resources
+        
+        self.package_name = package_name
         self.package_path = package_path
 
-    def locate(self, resource):
+    def locate(self, resource: str) -> str:
         pieces = split_resource_path(resource)
-        p = '/'.join((self.package_path,) + tuple(pieces))
-        if not self.provider.has_resource(p):
+        
+        try:
+            # Try to access the resource
+            package = self.resources.files(self.package_name)
+            if self.package_path:
+                package = package / self.package_path
+            
+            for piece in pieces:
+                package = package / piece
+            
+            if not package.is_file():
+                raise ResourceNotFound(resource)
+            
+            # For Python 3.9+, we need to handle the path properly
+            if hasattr(package, '__fspath__'):
+                return str(package)
+            else:
+                # Use as_file context manager for temporary access
+                with self.resources.as_file(package) as path:
+                    return str(path)
+        except (AttributeError, FileNotFoundError, ModuleNotFoundError):
             raise ResourceNotFound(resource)
 
-        return self.provider.get_resource_filename(self.manager, p)
-
 class FunctionLocator(BaseLocator):
     """A locator that is passed a function which does the searching.  The
     function becomes the name of the resource passed and has to return
@@ -169,4 +202,3 @@ class ChoiceLocator(BaseLocator):
                 pass
 
         raise ResourceNotFound(resource)
-
